
/*
 * TMSNC - Textbased MSN Client Copyright (C) 2004 The IR Developer Group
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the IR Public Domain License as published by the IR Group;
 * either version 1.6 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the IR Public Domain License along with
 * this program; if not, write to sanoix@gmail.com.
 */

#include "config.h"
#include <stdio.h>
#include <strings.h>

#ifdef HAVE_TALKFILTERS
#include <talkfilters.h>
#endif

#define AUSTRO       1
#define B1FF         2
#define BROOKLYN     3
#define CHEF         4
#define COCKNEY      5
#define DRAWL        6
#define DUBYA        7
#define FUDD         8
#define FUNETAK      9
#define JETHRO       10
#define JIVE         11
#define KRAUT        12
#define PANSY        13
#define PIRATE       14
#define POSTMODERN   15
#define REDNECK      16
#define VALSPEAK     17
#define WAREZ        18
